package com.bergerkiller.bukkit.nolagg;

import java.lang.reflect.Field;
import java.util.HashSet;
import java.util.Set;
import java.util.logging.Level;

import org.bukkit.Bukkit;
import org.bukkit.craftbukkit.CraftServer;

import net.minecraft.server.Chunk;
import net.minecraft.server.ChunkProviderServer;
import net.minecraft.server.IChunkLoader;
import net.minecraft.server.World;
import net.minecraft.server.WorldServer;

@SuppressWarnings("rawtypes")
public class ChunkSaving {

	private static DummyWorld dummyWorld;
	private static Field chunkloader;
	private static Field ticklist;
	public static boolean enabled = false;
	
	public static CraftServer getServer() {
		return (CraftServer) Bukkit.getServer();
	}
			
	public static void init() {
		try {
            dummyWorld = new DummyWorld();
			chunkloader = ChunkProviderServer.class.getDeclaredField("e");
			ticklist = World.class.getDeclaredField("L");
			chunkloader.setAccessible(true);
			ticklist.setAccessible(true);
			enabled = true;
		} catch (Throwable t) {
			NoLagg.log(Level.SEVERE, "Failed to initialize async chunk saving implementation:");
			t.printStackTrace();
			enabled = false;
		}
	}
	public static void deinit() {
		enabled = false;
		dummyWorld = null;
		ticklist = null;
		chunkloader = null;
	}
	
	@SuppressWarnings("unchecked")
	public static Set getTickList(WorldServer from) {
		if (ticklist == null) return null;
		try {
			Set source = (Set) ticklist.get(from);
			Set ticks = new HashSet(source.size());
			ticks.addAll(source);
			return ticks;
		} catch (Exception e) {
			NoLagg.log(Level.SEVERE, "Failed to obtain tick lists from world:");
			e.printStackTrace();
			ticklist = null;
			return null;
		}
	}
	public static void setDummyTickList(Set ticks) {
		if (ticklist == null) return;
		try {
			ticklist.set(dummyWorld, ticks);
		} catch (Exception e) {
			NoLagg.log(Level.SEVERE, "Failed to apply tick lists to dummy world:");
			e.printStackTrace();
			ticklist = null;
		}
	}
	
	public static void saveChunk(final Chunk chunk, Set ticks) {
		if (!enabled || chunk == null) return;
		try {
			//prepare entities
			ChunkHandler.fixEntities(chunk);
			//prepare dummy
			dummyWorld.setTime(chunk.world.getTime());
			setDummyTickList(ticks);
			//dummyWorld used for saving
			IChunkLoader loader = (IChunkLoader) chunkloader.get(((WorldServer) chunk.world).chunkProviderServer);
			loader.a(dummyWorld, chunk);
		} catch (Exception e) {
			NoLagg.log(Level.WARNING, "Failed to save chunk " + ChunkHandler.toString(chunk) + ":");
			e.printStackTrace();
		}
		//deinit dummy world
		setDummyTickList(null);
	}

}
